using CRMPluginUtils;
using Microsoft.Win32;
using MicrosoftOutlookServiceProvider;
using MyPhonePlugins;
using System;
using System.IO;
using System.Net;
using System.ServiceModel;

namespace MicrosoftOutlookPlugin
{
  public class OutlookPlugin : AbsCallNotifier
  {
    private static object lockObj = new object();

    private ConfigurationManager configurationManager = null;

    private ServiceHost serviceHost = null;

    private void outlookMakeCall(ContactInfo contactInfo)
    {
      MakeCall(contactInfo);
    }

    public void OnMyPhoneStatusChanged(IExtensionInfo extensionInfo, MyPhoneStatus status)
    {
      onMyPhoneStatusChanged(extensionInfo, status);
    }

    public void OnCallStatusChanged(CallStatus callInfo)
    {
      onCallStatusChanged(null, callInfo);
    }

    public OutlookPlugin(IMyPhoneCallHandler callHandler)
    : base("MicrosoftOutlookPlugin.log", callHandler)
    {

      ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
      this.configurationManager = new ConfigurationManager("3CXCRMUser.ini");
      OutboundCallRequestManager.OnOutboundCallRequest += new OutboundCallRequestManager.OutboundCallRequestHandler(outlookMakeCall);
      NetNamedPipeBinding binding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
      Uri baseAddress = new Uri("net.pipe://localhost/OutboundCallRequestServiceProvider_" + OwnerFinder.GetCurrentProcessOwnerId());
      LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Registering server binding to accept connections from Outlook add-in using address: " + baseAddress.OriginalString);
      serviceHost = new ServiceHost(typeof(OutboundCallRequestServiceProvider), baseAddress);
      serviceHost.AddServiceEndpoint(typeof(IOutboundCallRequestServiceProvider), binding, baseAddress);
      serviceHost.Open();

      configurationManager.SetValue("Microsoft Outlook Plugin", "IsRunning", "True");
      OutlookHelper.RegisterAddin();
    }

    ~OutlookPlugin()
    {
      lock (lockObj)
      {
        if (configurationManager != null)
	        configurationManager.SetValue("Microsoft Outlook Plugin", "IsRunning", "False");
      }
    }

    protected override ContactInfo onShowContact(string contactNumber, bool createIfNotFound)
    {
      if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "onShowContactRecord for contactNumber=" + contactNumber);
      lock (lockObj)
      {
        if (configurationManager.GetValue("Microsoft Outlook Add-in", "IsRunning", "False") != "True")
          throw new ApplicationException(LocalizedResourceManager.GetString("DotNetScript", "OutlookPlugin.Error.OutlookAddinIsNotRunning"));

        try
        {
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Getting Service Provider");
          IOutlookServiceProvider serviceProvider = OutlookHelper.GetServiceProvider();
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Invoking add-in method via RPC");
          serviceProvider.ShowContactRecord(contactNumber, createIfNotFound);
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "RPC method invocation complete");
          return null;
        }
        catch (FaultException e)
        {
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "RPC method invocation error - FaultException: " + ErrorHelper.GetErrorDescription(e));
          throw new ApplicationException("Error connecting to Microsoft Outlook.", e);
        }
      }
    }

    protected override void onStoreCallInformation(CallInformation callInformation)
    {
      if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "onStoreCallInformation for contactNumber=" + callInformation.ContactNumber);
      lock (lockObj)
      {
        if (configurationManager.GetValue("Microsoft Outlook Add-in", "IsRunning", "False") != "True")
          throw new ApplicationException(LocalizedResourceManager.GetString("DotNetScript", "OutlookPlugin.Error.OutlookAddinIsNotRunning"));

        try
        {
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Getting Service Provider");
          IOutlookServiceProvider serviceProvider = OutlookHelper.GetServiceProvider();
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "Invoking add-in method via RPC");
          serviceProvider.StoreCallInformation(callInformation);
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "RPC method invocation complete");
        }
        catch (FaultException e)
        {
          if (GeneralConfiguration.IsDebugEnabled) LogHelper.Log(Environment.SpecialFolder.ApplicationData, "MicrosoftOutlookPlugin.log", "RPC method invocation error - FaultException: " + ErrorHelper.GetErrorDescription(e));
          throw new ApplicationException("Error connecting to Microsoft Outlook.", e);
        }
      }
    }
  }
}
